Package: jackstraw
Type: Package
Title: Statistical Inference for Unsupervised Learning
Version: 1.3.21
Authors@R: c(
    person(given = c("Neo", "Christopher"),
           family = "Chung",
           role = c("aut", "cre"),
           email = "nchchung@gmail.com",
           comment = c(ORCID = "0000-0001-6798-8867")),
    person(given = c("John", "D."),
           family = "Storey",
           role = "aut",
           email = "jstorey@princeton.edu",
           comment = c(ORCID = "0000-0001-5992-402X")),
    person(given = "Wei",
           family = "Hao",
           role = "aut",
           email = "whao@princeton.edu"),
    person(given = "Alejandro",
           family = "Ochoa",
           role = "aut",
           email = "alejandro.ochoa@duke.edu",
           comment = c(ORCID = "0000-0003-4928-3403"))
    )
Description: Test for association between the observed data and their estimated latent variables. The jackstraw package provides a resampling strategy and testing scheme to estimate statistical significance of association between the observed data and their latent variables. Depending on the data type and the analysis aim, the latent variables may be estimated by principal component analysis (PCA), factor analysis (FA), K-means clustering, and related unsupervised learning algorithms. The jackstraw methods learn over-fitting characteristics inherent in this circular analysis, where the observed data are used to estimate the latent variables and used again to test against that estimated latent variables. When latent variables are estimated by PCA, the jackstraw enables statistical testing for association between observed variables and latent variables, as estimated by low-dimensional principal components (PCs). This essentially leads to identifying variables that are significantly associated with PCs. Similarly, unsupervised clustering, such as K-means clustering, partition around medoids (PAM), and others, finds coherent groups in high-dimensional data. The jackstraw estimates statistical significance of cluster membership, by testing association between data and cluster centers. Clustering membership can be improved by using the resulting jackstraw p-values and posterior inclusion probabilities (PIPs), with an application to unsupervised evaluation of cell identities in single cell RNA-seq (scRNA-seq).
LazyData: true
Depends: R (>= 3.0.0)
biocViews:
Imports: methods, stats, stats4, corpcor, irlba, rsvd, ClusterR,
        cluster, BEDMatrix, genio (>= 1.0.15.9000)
Suggests: qvalue, lfa (>= 2.0.6.9000), gcatest (>= 2.0.4.9000),
        testthat (>= 3.0.0)
License: GPL-2
Encoding: UTF-8
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-21 18:46:26 UTC; ncc
Author: Neo Christopher Chung [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6798-8867>),
  John D. Storey [aut] (ORCID: <https://orcid.org/0000-0001-5992-402X>),
  Wei Hao [aut],
  Alejandro Ochoa [aut] (ORCID: <https://orcid.org/0000-0003-4928-3403>)
Maintainer: Neo Christopher Chung <nchchung@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-22 07:20:02 UTC
Built: R 4.6.0; ; 2026-04-26 19:15:32 UTC; unix
