# Generated by roxygen2: do not edit by hand

export(find_k)
export(jackstraw_MiniBatchKmeans)
export(jackstraw_alstructure)
export(jackstraw_cluster)
export(jackstraw_irlba)
export(jackstraw_kmeans)
export(jackstraw_kmeanspp)
export(jackstraw_lfa)
export(jackstraw_pam)
export(jackstraw_pca)
export(jackstraw_rpca)
export(jackstraw_subspace)
export(ncp_est)
export(permutationPA)
export(pip)
export(pvals_nc_chisq)
